import { s } from './h.js'

const COLORS = ['#e22', '#e73', '#fc3', '#ad4', '#4d9', '#3be', '#45d', '#c3e']

/**
 * A helper for creating a chart based on a table (conf. <https://jackens.github.io/nnn/chartable/>).
 *
 * Options:
 * - `bottom`: bottom padding (for X axis labels)
 * - `gapX`: X axis spacing
 * - `gapY`: Y axis spacing
 * - `headerColumn`: flag indicating that `table` has a header column (with X axis labels)
 * - `id`: chart id
 * - `left`: left padding
 * - `maxY`: number of Y axis lines
 * - `reverse`: flag to reverse all data series
 * - `right`: right padding (for data series labels)
 * - `rotate`: flag to rotate X axis labels
 * - `table`: `HTMLTableElement` to extract data, data series labels and X axis labels
 * - `top`: top padding
 *
 * @param {{
 *   bottom?: number;
 *   gapX?: number;
 *   gapY?: number;
 *   headerColumn?: boolean;
 *   id?: string;
 *   left?: number;
 *   maxY?: number;
 *   reverse?: boolean;
 *   right?: number;
 *   rotate?: boolean;
 *   table: HTMLTableElement;
 *   top?: number;
 * }} options
 */
export const chartable = ({
  bottom = 50,
  gapX = 70,
  gapY = 30,
  headerColumn = false,
  id,
  left = 15,
  maxY = 10,
  reverse = false,
  right = 200,
  rotate = false,
  table,
  top = 15
}) => {
  const /** @type {number[][]} */ zxY = []
  const /** @type {string[]}   */ xLabels = []
  const /** @type {string[]}   */ zLabels = []

  table.querySelectorAll('tr').forEach((row, r) =>
    // @ts-expect-error
    row.querySelectorAll('td,th').forEach((/** @type {HTMLTableCellElement} */ col, c) => {
      const x = r - 1
      const z = c - +headerColumn
      const value = col.innerText

      if (x >= 0 && z >= 0) {
        zxY[z] = zxY[z] ?? []
        zxY[z][x] = parseFloat(value)
      } else if (x >= 0 && z < 0) {
        xLabels[x] = value
      } else if (x < 0 && z >= 0) {
        zLabels[z] = value
      }
    }))

  if (reverse) {
    xLabels.reverse()
    zxY.forEach(xY => xY.reverse())
  }

  const maxX = zxY[0].length
  const xi = Array.from({ length: maxX }, (_, x) => x * gapX)
  const yi = Array.from({ length: maxY }, (_, y) => y * gapY)
  const w = gapX * (maxX - 1)
  const h = gapY * (maxY - 1)
  const svgW = left + w + right
  const svgH = top + h + bottom
  const /** @type {import('./h.js').HArgs1} */ graph = ['g', { transform: `translate(${left} ${top})` },
    ...yi.map(y => ['line', { x1: 0, x2: w, y1: y, y2: y, stroke: '#8888' }]),
    ...xi.map(x => ['line', { x1: x, x2: x, y1: 0, y2: h, stroke: '#8888' }])]
  const /** @type {import('./h.js').HArgs1} */ colors = ['g', { transform: `translate(${left + w + 25} ${top + 15})` }]
  const /** @type {import('./h.js').HArgs1} */ labels = ['g', { transform: `translate(${left + w + 40} ${top + 15})` }]

  zxY.forEach((xY, z) => {
    const fill = COLORS[z % COLORS.length]
    const min = Math.min(...xY.filter(x => !isNaN(x)))
    const max = Math.max(...xY.filter(x => !isNaN(x)))

    colors.push(['circle', { cx: 0, cy: gapY * (colors.length - 2), r: 5, fill }])
    labels.push(['text',
      { x: 0, y: gapY * (labels.length - 2), 'text-anchor': 'start', 'alignment-baseline': 'middle' },
      zLabels[z]])

    const y = xY.map(x => isNaN(x) ? x : h * (max - x) / (max - min))

    xi.forEach((x, i) => {
      if (!isNaN(y[i])) {
        graph.push(['g', ['circle', { cx: x, cy: y[i], r: 5, fill }, ['title', `${zLabels[z]}: ${xY[i]}`]]])

        if (!isNaN(y[i - 1])) {
          graph.push(['line', { x1: x, x2: xi[i - 1], y1: y[i], y2: y[i - 1], stroke: fill }])
        }
      }
    })
  })

  return s('svg', {
    viewBox: `0 0 ${svgW} ${svgH}`,
    width: `${svgW}px`,
    height: `${svgH}px`,
    class: 'chartable'
  }, id != null ? { id } : null,
  graph, colors, labels,
  xLabels.length > 0
    ? ['g', { transform: `translate(${left} ${top + h + 15})` },
        ...xi.slice(0).map((x, i) => ['text', rotate
          ? { x: 0, y: -x, 'text-anchor': 'start', 'alignment-baseline': 'hanging', transform: 'rotate(90)' }
          : { x, y: 0, 'text-anchor': 'middle', 'alignment-baseline': 'hanging' },
        xLabels[i]])]
    : null
  )
}

export const tests = {}
